@@ -6,6 +6,10 @@ import logging
 
 from copy import copy
 
+from schedules_tools import SchedulesToolsException
+
+from .utils import sort_tasks, slugify
+
 log = logging.getLogger(__name__)
 re_flags_separator = re.compile('[, ]+')
 
@@ -13,6 +17,10 @@ ATTR_PREFIX_FLAG = 'Flags'
 ATTR_PREFIX_LINK = 'Link'
 ATTR_PREFIX_NOTE = 'Note'
 
+TASK_SLUG_NUMERATION_LIMIT = 9999
+TASK_SLUG_CHAR_LIMIT = 255
+TASK_SLUG_TRUNCATION_LIMIT = TASK_SLUG_CHAR_LIMIT - (1 + len(str(TASK_SLUG_NUMERATION_LIMIT)))
+
 
 class Task(object):
     index = ''
@@ -22,8 +30,6 @@ class Task(object):
     priority = 500
     dStart = datetime.datetime.max
     dFinish = datetime.datetime.min
-    dAcStart = datetime.datetime.max
-    dAcFinish = datetime.datetime.min
     milestone = False
     p_complete = 0.0
     flags = []
@@ -49,9 +55,9 @@ class Task(object):
         self.level = level
 
     def __unicode__(self):
-        return '%s %s MS:%s  (%s - %s) ac(%s - %s)  F%s  [%s]' % (
+        return '%s %s MS:%s  (%s - %s)  F%s  [%s]' % (
             self.slug, self.name, self.milestone, self.dStart, self.dFinish,
-            self.dAcStart, self.dAcFinish, self.flags, len(self.tasks))
+            self.flags, len(self.tasks))
 
     def __str__(self):
         return unicode(self).encode('utf-8')
@@ -70,7 +76,8 @@ class Task(object):
 
         if key:
             val = value
-        else:
+        elif hasattr(value, 'split'):
+            # test value for flag / note / link
             pieces = value.split(':', 1)
             if len(pieces) != 2:
                 # it's not a string in format 'Flag: qe, dev' - don't process
@@ -80,8 +87,11 @@ class Task(object):
             key = key.strip().lower()
             val = val.strip()
 
+        else:
+            return
+
         if key.lower() == ATTR_PREFIX_FLAG.lower():
-            val = val.lower()
+            val = str(val).lower()
             flags = re_flags_separator.split(val)
             for flag in flags:
                 flag = str(flag.strip())
@@ -128,13 +138,6 @@ class Task(object):
             return name
         return ''
 
-    def check_for_phase(self):
-        PHASE_NAMES = ('planning phase', 'development phase', 'testing phase',
-                       'launch phase', 'maintenance phase')
-        if self.name.lower() in PHASE_NAMES:
-            self.name = self.name.title()
-            self._schedule.phases.append(self)
-
     @property
     def desc_tasks_count(self):
         count = len(self.tasks)
@@ -145,7 +148,7 @@ class Task(object):
     def get_type(self, check_same_start_finish=False):
         if self.tasks:
             return 'Container'
-        elif self.milestone and (not check_same_start_finish or self.dAcStart == self.dAcFinish):
+        elif self.milestone and (not check_same_start_finish or self.dStart == self.dFinish):
             return 'Milestone'
         else:
             return 'Task'
@@ -181,17 +184,28 @@ class Task(object):
 
     @property
     def subtree_hash(self):
-        attrs = ['name', 'dStart', 'dFinish', 'dAcStart', 'dAcFinish']
+        attrs = ['name', 'dStart', 'dFinish']
 
         if self._subtree_hash is None:
             self._subtree_hash = ''
 
             for child_task in self.tasks:
-                values_list = [str(getattr(child_task, attr)) for attr in attrs]
+                values_list = [unicode(getattr(child_task, attr)) for attr in attrs]
                 self._subtree_hash += ''.join(values_list) + child_task.subtree_hash
 
         return self._subtree_hash
 
+    def get_slug_key(self, prefix=None):
+        """
+        Generates slug key only for the task itself.
+        Doesn't include parent tasks slugs i.e. slug key is not a full slug path to the task.
+        """
+        slug_key = slugify(self.name)
+
+        if prefix is not None:
+            return '.'.join((prefix, slug_key))
+        return slug_key
+
 
 class Schedule(object):
     slug = ''
@@ -202,7 +216,6 @@ class Schedule(object):
     changelog = {}
     mtime = None
 
-    phases = []
     resources = {}
     assignments = []
     used_flags = None
@@ -218,13 +231,13 @@ class Schedule(object):
 
     def __init__(self):
         self.tasks = []
-        self.phases = []
         self.used_flags = set()
         self.changelog = {}
         self.ext_attr = {}
         self.unique_id_re = re.compile('[\W_]+')
         self.errors_import = []
         self.mtime = None
+        self.tasks_slugs = None
 
 
     def make_flat(self):
@@ -279,36 +292,39 @@ class Schedule(object):
 
                 # Add to result if not hidden and (has subtasks or fits show)
                 logmsg_fmt = 'REMOVE: {} {}'
-                if (not (set(task.flags) & set(hide)) 
-                    and (filtered_subtasks 
+                if (not (set(task.flags) & set(hide))
+                    and (filtered_subtasks
                          or (set(task.flags) & set(show))
                          or not show)):
                     task.tasks = filtered_subtasks
                     tasks_list.append(task)
-                    
+
                     logmsg_fmt = 'ADD: {} {}'
-                    
+
                     self.used_flags.update(task.flags)
-                
+
                 log.info(logmsg_fmt.format(task.name, task.flags))
-                
-            
+
             return tasks_list
-            
+
         if show is None:
             show = []
         if hide is None:
             hide = []
-            
+
         if show or hide:
             self.used_flags = set()
-            
+
             log.info('FLAG filter: SHOW {}   HIDE {}'.format(show, hide))
 
             self.tasks = filter_tasks(self.tasks)
 
+    def sort_tasks(self, field):
+        self.tasks = sort_tasks(self.tasks, field)
 
     def check_top_task(self):
+        """ Add single root task if multiple """
+
         def _raise_index(task):
             task.index = int(task.index) + 1
             for t in task.tasks:
@@ -320,7 +336,7 @@ class Schedule(object):
         # need one top task
         top_task = Task(self)
         top_task.index = 1
-        #top_task.name = '%s %s' % (self.name, self.version)
+        # top_task.name = '%s %s' % (self.name, self.version)
         # removed version as per BZ#1396303 - see how that works
         top_task.name = self.name
         top_task.slug = self.slug
@@ -328,43 +344,53 @@ class Schedule(object):
         for task in self.tasks:
             if top_task.dStart:
                 top_task.dStart = min(top_task.dStart,
-                                      task.dStart,
-                                      task.dAcStart)
+                                      task.dStart)
             else:
                 top_task.dStart = task.dStart
 
-            top_task.dAcStart = top_task.dStart
-
             if top_task.dFinish:
                 top_task.dFinish = max(top_task.dFinish,
-                                       task.dAcFinish,
                                        task.dFinish)
             else:
                 top_task.dFinish = task.dFinish
 
-            top_task.dAcFinish = top_task.dFinish
-
             _raise_index(task)
             top_task.tasks.append(task)
 
         self.tasks = [top_task]
-        
-    
+
     def generate_slugs(self):
-        def gen_slugs_recurse(tasks, id_prefix):
+        def gen_slugs_recursive(tasks, slug_prefix=None):
             for task in tasks:
-                task.slug = self.get_unique_id(task.name, id_prefix)
-                gen_slugs_recurse(task.tasks, task.slug)
-        
-        self.id_reg = set()
-        gen_slugs_recurse(self.tasks, self.slug)
-    
+                task_slug = task.get_slug_key(slug_prefix)
+
+                if task_slug > TASK_SLUG_TRUNCATION_LIMIT:
+                    task_slug = task_slug[:TASK_SLUG_TRUNCATION_LIMIT]
+
+                if task_slug in self.tasks_slugs:
+                    for i in xrange(2, TASK_SLUG_NUMERATION_LIMIT + 1):
+                        task_slug_numerated = '%s_%d' % (task_slug, i)
+                        if task_slug_numerated not in self.tasks_slugs:
+                            final_task_slug = task_slug_numerated
+                            break
+                        elif i == TASK_SLUG_NUMERATION_LIMIT:
+                            raise SchedulesToolsException('Task slug generation error: '
+                                                          'reached numeration limit')
+                else:
+                    final_task_slug = task_slug
+
+                task.slug = final_task_slug
+                self.tasks_slugs.add(task.slug)
+                gen_slugs_recursive(task.tasks, task.slug)
+
+        self.tasks_slugs = set()
+        gen_slugs_recursive(self.tasks)
 
     def print_tasks(self, tasks=None, level=0):
         if tasks is None:
             tasks = self.tasks
         for task in tasks:
-            print level * '  ', str(task)
+            print(level * '  ', str(task))
             if task.tasks:
                 self.print_tasks(task.tasks, level + 1)
 
@@ -391,8 +417,7 @@ class Schedule(object):
         return diff
 
     def _diff_tasks(self, left, right, attrs=None, whole_days=False):
-        default_attrs = ['name', 'dStart', 'dFinish', 'dAcStart', 'dAcFinish',
-                         'link', 'note']
+        default_attrs = ['name', 'dStart', 'dFinish', 'link', 'note']
         if not attrs:
             attrs = default_attrs
         ret = ''
@@ -491,44 +516,6 @@ class Schedule(object):
     def slugify_str(self, orig_str):
         return self.unique_id_re.sub('_', orig_str.lower())
 
-
-    def get_unique_id(self, orig_str, id_prefix=''):
-        '''Return unique id within schedule'''
-        
-        # shortcut - first orig_str gets prefix        
-        if not self.id_reg and id_prefix:
-            self.id_reg.add(id_prefix)
-            return id_prefix
-
-        pref = copy(id_prefix)
-        pref += '.'
-
-        source = self.slugify_str(orig_str)
-
-        found = False
-        test_id = ''
-        for word in source.split('_'):
-            test_id = pref + word
-            if test_id in self.id_reg:
-                pref = test_id + '_'
-            else:
-                found = True
-                self.id_reg.add(test_id)
-                break
-
-        if not found:
-            # duplicate orig_str names - add numbering
-            n = 2
-            while '%s_%s' % (test_id, n) in self.id_reg:
-                n += 1
-
-            test_id = '%s_%s' % (test_id, n)
-            self.id_reg.add(test_id)
-
-            log.info('Duplicate Names: %s, adding: %s' % (source, test_id))
-
-        return test_id
-
     def dump_as_dict(self):
         schedule = copy(vars(self))
 
@@ -536,10 +523,6 @@ class Schedule(object):
         for task in self.tasks:
             schedule['tasks'].append(task.dump_as_dict())
 
-        schedule['phases'] = []
-        for phase in self.phases:
-            schedule['phases'].append(phase.dump_as_dict(recursive=False))
-
         # set() is not serializable into json
         schedule['used_flags'] = sorted(list(self.used_flags))
         return schedule
@@ -555,10 +538,6 @@ class Schedule(object):
         for task in input_dict['tasks']:
             schedule.tasks.append(Task.load_from_dict(task, schedule))
 
-        schedule.phases = []
-        for phase in input_dict['phases']:
-            schedule.phases.append(Task.load_from_dict(phase, schedule))
-
         schedule.used_flags = set(input_dict['used_flags'])
 
         return schedule
