@@ -1,5 +1,6 @@
 import datetime
 import logging
+import numbers
 import re
 import traceback
 
@@ -44,7 +45,7 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
     _client_instance = None
     _sheet_instance = None
     _re_number = re.compile('[0-9]+')
-    _re_url = re.compile('^https?://.*?smartsheet.com')
+    _re_url = re.compile(r'^(https?://.*?smartsheet.com[^?]+)')
     columns_mapping_name = {
         'Task': COLUMN_TASK_NAME,
         'Task Name': COLUMN_TASK_NAME,
@@ -92,14 +93,18 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
             except smartsheet.exceptions.ApiError as e:
                 log.warn(e)
 
-        elif ScheduleHandler_smartsheet.value_is_ss_permalink(value):
-            sheets = self.client.Sheets.list_sheets(include_all=True)
+        else:
+            match = ScheduleHandler_smartsheet.value_is_ss_permalink(value)
+            if match:
+                value = match.groups()[0]
 
-            for sheet in sheets.data:
-                if sheet.permalink == value:
-                    info_dict = info_dict = dict(id=int(sheet.id),
-                                                 permalink=value)
-                    break
+                sheets = self.client.Sheets.list_sheets(include_all=True)
+
+                for sheet in sheets.data:
+                    if sheet.permalink == value:
+                        info_dict = info_dict = dict(id=int(sheet.id),
+                                                     permalink=value)
+                        break
 
         return info_dict
 
@@ -114,9 +119,7 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
 
     @classmethod
     def value_is_ss_permalink(cls, value):
-        # https://app.smartsheet.com/b/home?lx=0HHzeGnfHik-N13ZT8pU7g
-        if cls._re_url.match(str(value)):
-            return True
+        return cls._re_url.match(str(value))
 
     @classmethod
     def is_valid_source(cls, handle=None):
@@ -168,6 +171,8 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
                 self.schedule.unique_id_re.sub('_', self.schedule.name))
             self.schedule.mtime = self.get_handle_mtime()
             self.schedule.changelog = self.get_handle_changelog()
+            self.schedule.dStart = datetime.datetime.max
+            self.schedule.dFinish = datetime.datetime.min
 
             parents_stack = []
 
@@ -182,9 +187,12 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
             for row in self.sheet.rows:
                 self._load_task(row, parents_stack)
 
-            self.schedule.check_top_task()
-            self.schedule.dStart = self.schedule.tasks[0].dStart
-            self.schedule.dFinish = self.schedule.tasks[0].dFinish
+            if not self.schedule.tasks:
+                self.schedule.dStart = None
+                self.schedule.dFinish = None
+                log.warning('Empty schedule (no valid tasks) %s'
+                            % self.get_info_dict(self.handle)['permalink'])
+
             self.schedule.generate_slugs()
 
         except (KeyError, IndexError):
@@ -230,7 +238,7 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
 
         # skip empty rows
         if not all([c in cells and cells[c]
-                    for c in COLUMN_TASK_NAME, COLUMN_START, COLUMN_FINISH]):
+                    for c in (COLUMN_TASK_NAME, COLUMN_START, COLUMN_FINISH)]):
             # skip load, task doesn't contain all needed info
             return
 
@@ -238,10 +246,10 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
 
         # dates
         task.dStart = self._parse_date(cells[COLUMN_START])
-        task.dAcStart = task.dStart
-
         task.dFinish = self._parse_date(cells[COLUMN_FINISH])
-        task.dAcFinish = task.dFinish
+
+        self.schedule.dStart = min(self.schedule.dStart, task.dStart)
+        self.schedule.dFinish = max(self.schedule.dFinish, task.dFinish)
 
         if COLUMN_NOTE in cells and cells[COLUMN_NOTE]:
             task.note = unicode(cells[COLUMN_NOTE])
@@ -255,7 +263,8 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
             if match:
                 task.milestone = int(match[0]) == 0
 
-        if COLUMN_P_COMPLETE in cells and cells[COLUMN_P_COMPLETE] is not None:
+        if (COLUMN_P_COMPLETE in cells and cells[COLUMN_P_COMPLETE] is not None
+                and isinstance(cells[COLUMN_P_COMPLETE], numbers.Number)):
             task.p_complete = round(cells[COLUMN_P_COMPLETE] * 100, 1)
 
         if COLUMN_FLAGS in cells and cells[COLUMN_FLAGS]:
@@ -304,8 +313,6 @@ class ScheduleHandler_smartsheet(ScheduleHandlerBase):
                 self.schedule.tasks.append(task)
 
             parents_stack.insert(0, curr_stack_item)
-
-        task.check_for_phase()
         task.level = len(parents_stack)
 
     def _load_task_cells(self, row):
